function [u,v,X,Y,Einbau] = UseCase_v4_iter(A,u0,v0)

    %% Erklrung
    % Du nimmst Dir ein Bild, rechtsklickst im Matlab-Browser darauf,
    % whlst import Data und finish. Dann hast du eine M-by-N-by-3-Matrix im
    % Workspace. Diese Matrix fgst Du hier als Argument A ein.
    %
    % u0 und v0 kannst Du einfgen, falls Du fr vorherige Strmungsdaten
    % weiterrechnen willst. Sonst lsst Du die Argumente einfach weg.
    %
    % Anschlieend musst Du noch einen Ordner erstellen, in dem die Bilder
    % deiner Simulation gespeichert werden sollen. Mit dem
    % Windows-Movie-Maker kannst Du spter aus der Bilderreihe einen Film
    % generieren.
    %
    % VG
    % Marin
    
    
    %% Matrix bearbeiten:
    A = (A(:,:,1) == 0);
    A = A .* ones(size(A));
    A = A(size(A,1):-1:1,:);
    A = A';
    

    %% Parameter
    M       = size(A,1);
    N       = size(A,2);
    K       = 1000;
    K0      = 0;
    dK      = 100;  % nach jedem m-ten Zeitschritt wird ein Bild ausgegeben
    tol     = 0.5e-1;    
    direkt  = true;
    Daten   = true;
    fix_dt  = true;
    
    myfigure = figure;
    
    fpat = 'Lauf1';
    
    %% Simulationsparameter
    u_in    = 1;
    v_in    = 0;
    ny      = 0;
    dx      = 1;
    dt      = 0.3;

    
    lambda = dt/dx;
    
    alpha = 1e-6;   % Tikhonov regularizator
    
    
    %% Einbau    
    Einbau = (A == 1);

    Null = zeros(M,N);
    Eins = ones(M,N);
    
    
    %% Anfangsverteilung
    if(nargin == 3)
        u = u0;
        v = v0;
    else
        %% Init
         u = 1*      ones(M,N);
         v = 0*      ones(M,N);   
    end
    
    
    %% Strmungscharakteristika
    ny_til  = dx^2/dt + ny;
    L       = min(M,N) * dx; 
    
    Re = sqrt(v_in^2 + u_in^2) * L / ny_til
    
    
    %% Randbedingungen
    % Matrix mit 1-Eintrgen fr Rnder
    S           = zeros(M,N);
    S(1,:)      = ones(1,N);
    S(M,:)      = ones(1,N);
    S(:,1)      = ones(M,1);
    S(:,N)      = ones(M,1);
    S(Einbau)   = Eins(Einbau);
    % Einstrom-Flchen
    u(1,:)      = 0 * ones(1,N);
    u(M,:)      = 0 * ones(1,N);
    v(1,:)      = 0 * ones(1,N);
    v(M,:)      = 0 * ones(1,N);
    % No-Slip-Rnder
    u(:,1)      = 0 * ones(M,1);
    u(:,N)      = 0 * ones(M,1);
    v(:,1)      = 1 * ones(M,1);
    v(:,N)      = 1 * ones(M,1);
    % Einbau
    u(Einbau)      = Null(Einbau);
    v(Einbau)      = Null(Einbau);
    
    % Die Randboolwerte  werden in einen Vektor berfhrt
    for i=1:M
        vS((i-1)*N+1:i*N,1)         = S(i,:)';
    end
    for i=1:M
        vS(M*N+(i-1)*N+1:M*N+i*N,1) = S(i,:)';
    end
    vNS = ones(2*M*N,1) - vS;
    vNS = (vNS == 1);
    
    A = A_init2(M,N);
    

    %% Lsungsmethode vorbereiten
    % variable Spalten extrahieren
    % Normalengleichung aufstellen
    % Gauss-Zerlegung der Normalenmatrix
    A_NS = sparse(A(:,vNS));
    tic
    Z = (A_NS * A_NS' + alpha * speye((M-1)*(N-1),(M-1)*(N-1))); 
    disp('Zeit fr Produkt');
    time = toc
    disp('Zeit fr Sparsifizierung')
    tic
    Z = sparse(Z);
    time = toc
    if(direkt)
        disp('Zeit fr Zerlegung')
        tic
        [L,U,P,Q] = lu(Z);
        %[R,P,p] = chol(Z);
        time = toc
    else
        disp('Zeit fr Prkonditionierung')
        tic
        [L] = ichol(Z,struct('type','ict','droptol',1e-03,'michol','on'));
        p = zeros(size(Z,1),1);
    end
    
    
    %% Hilfsvariablen
    mit_u       = zeros(size(u));
    mit_v       = zeros(size(v));
    x_u         = zeros(size(u));
    x_v         = zeros(size(v));
    y_u         = zeros(size(u));
    y_v         = zeros(size(v));
    c           = zeros(2*M*N,1);
    notANumber  = NaN(size(u));
    
    sum_time = 0;
    %% Zeitschritte
    for k = 1:K
        disp('-----------------------')
        tic
%         u_in    = u_in + 0.0001;
%         v_in    = v_in + 0.0001;
        
        str = sprintf('Schritt %i, dt = %d',k,dt);
        
        disp(str);
        
        
        %% Hilfsvariablen berechnen
        % x-Ableitung
        x_u(2:M-1,2:N-1)        = (0.5/dx) * ( -u(1:M-2,2:N-1)+u(3:M,2:N-1) );
        x_v(2:M-1,2:N-1)        = (0.5/dx) * ( -v(1:M-2,2:N-1)+v(3:M,2:N-1) );
        
        % y-Ableitung
        y_u(2:M-1,2:N-1)        = (0.5/dx) * ( -u(2:M-1,1:N-2)+u(2:M-1,3:N) );
        y_v(2:M-1,2:N-1)        = (0.5/dx) * ( -v(2:M-1,1:N-2)+v(2:M-1,3:N) );
        
        % Mittel der vier Nachbarn
        mit_u(2:M-1,2:N-1)      = 0.25 * ( u(1:M-2,2:N-1)+u(3:M,2:N-1)+u(2:M-1,1:N-2)+u(2:M-1,3:N) );
        mit_v(2:M-1,2:N-1)      = 0.25 * ( v(1:M-2,2:N-1)+v(3:M,2:N-1)+v(2:M-1,1:N-2)+v(2:M-1,3:N) );
                
        % Mittel der vier Nachbarn
        mit_u(2:M-1,2:N-1)      = 0.25 * ( u(1:M-2,2:N-1)+u(3:M,2:N-1)+u(2:M-1,1:N-2)+u(2:M-1,3:N) );
        mit_v(2:M-1,2:N-1)      = 0.25 * ( v(1:M-2,2:N-1)+v(3:M,2:N-1)+v(2:M-1,1:N-2)+v(2:M-1,3:N) );
        
        
        %% Zeitschritt Strmung
        u = mit_u + dt * (- mit_u .* x_u - mit_v .* y_u);
        v = mit_v + dt * (- mit_u .* x_v - mit_v .* y_v);
        
        
        %% Randbedingungen
        % kann man rauskommentieren, da Randwerte gar nicht berschrieben
        % werden
        % -> kann man doch nicht rauskommentieren, da sie in der
        % Druckkorrektur berschrieben werden
        % Oberer und unterer Rand
        u(1,:)      = 1 * ones(1,N);
        v(1,:)      = 0 * ones(1,N);
        if(0)
            u(M,:)  = u(M-1,:);     % Neumann
            v(M,:)  = v(M-1,:);
        else
            u(M,:)  = 1 * ones(1,N);
            v(M,:)  = 0 * ones(1,N);
        end
        % Linker und rechter Rand
        u(:,1)      = u(:,N-1);%
        u(:,N)      = u(:,2);%
        v(:,1)      = v(:,N-1);
        v(:,N)      = v(:,2);%1 * ones(M,1);
        % Einbau
        u(Einbau)      = Null(Einbau);
        v(Einbau)      = Null(Einbau);

        
        %% Divergenzbereinigung
        for i=1:M
            c((i-1)*N+1:i*N,1) = u(i,:)';
        end
        for i=1:M
            c(M*N+(i-1)*N+1:M*N+i*N,1) = v(i,:)';
        end
        r = A * c;
        
        if(direkt)
            dc = A_NS' * ( - Q * ( U\ ( L\ (P*r) ) ) );
        else
            % Iterative Lsung
            p = pcg(Z,r,1e-6,50,L,L',p);
            dc = A_NS' * p;
        end
        % Update um LS-Minimalkorrektur
        c(vNS) = c(vNS) + dc;
        % von Vektor nach Matrix
        for i=1:M
            u(i,:) = c((i-1)*N+1:i*N,1)';
        end
        for i=1:M
            v(i,:) = c(M*N+(i-1)*N+1:M*N+i*N,1)';
        end
        
        
        %% Zeitprognose
        disp('Zeit fr diesen Schritt');
        time = toc
        disp('geschtzte Restzeit')
        sum_time = sum_time + time;
        Restzeit = sum_time/k * (K-k)
        
        
        %% Zeitschrittfeld plotten
        if(mod(k,dK) == 0)
            if(true)
                str =  sprintf('Schritt %i, dt = %d',k+K0,dt);            
                %% Fr Detail-Ansichten
                m1 = 1;
                m2 = M;
                m  = M-1;%ceil(M/5);
                n1 = 1;
                n2 = N;
                n  = N-1;%ceil(N/5);

                clf;
                %colormap((jet+white)/2);
                %colormap(summer);
                colormap(jet);
                hold on;
                grid off;
                axis equal;
                %title(str);
                xlabel('x');
                ylabel('y');



                [X,Y] = meshgrid(m1:ceil((m2-m1)/m):m2, n1:ceil((n2-n1)/n):n2);
                norm_u = ((u(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2) .* u(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2) + v(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2) .* v(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2)).^0.5);
                norm_u(Einbau) = notANumber(Einbau);

                %pcolor(X,Y,norm_u');

                pcolor(Y',X',norm_u(M:-1:1,:));

                %quiver(Y',X',v,u);
    %             axis off
    %             
    %             iptsetpref('ImshowBorder','tight');
    %             set(gca,'LooseInset',get(gca,'TightInset'))
    %             streamline(Y',X',v,u,startx,starty)


                set(gca,'position',[0 0 1 1],'units','normalized')
                axis off

                %shading interp;
                shading flat;
                axis tight;
                if(false)
                    u_t = 0.6 * u(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2) ./ norm_u;
                    v_t = 0.6 * v(m1:ceil((m2-m1)/m):m2,n1:ceil((n2-n1)/n):n2) ./ norm_u;
                    u_t(1,1) = 1;
                    quiver(X,Y,u_t',v_t','black')
                end

                str = sprintf('%i',k+K0);
                %saveas(gcf,[fpat,filesep,str],'jpg');
                 print(myfigure,'-djpeg','-r500',[fpat,filesep,str]);
    %            print(myfigure,'-djpeg','-r100',[fpat,filesep,str]);
            else
               %figure;
               clf;
                colormap((0*jet+4*bone)/4);
                %colormap((hot+white)/2)
                hold on;
                grid off;
                axis equal;
                
                [X,Y] = meshgrid(1:M,1:N);
                norm_u          = (u .* u + v .* v).^0.5;
                
                pcolor(Y',X',norm_u);

                set(gca,'position',[0 0 1 1],'units','normalized')
                axis off

                shading interp;

                
                
                
                u = u ./norm_u;
                v = v ./norm_u;

                
                N
                M
                size(X)
                size(Y)
                size(u)
                size(v)
                Dx = 10;
                %quiver(Y(1:Dx:N,1:Dx:M)',X(1:Dx:N,1:Dx:M)',v(1:Dx:M,1:Dx:N),u(1:Dx:M,1:Dx:N),'black');
                %quiver(Y(1:dx:N,1:dx:M)',X(1:dx:N,1:dx:M)',v(M:-dx:1,1:dx:N),-u(M:-dx:1,1:dx:N),'black');

                streamslice(Y',X',v,u);%,'black','linewidth',2);

                spy(Einbau,'green*');
                
                str = sprintf('%i',k+K0);
                %saveas(gcf,[fpat,filesep,str],'jpg');
                 print(myfigure,'-djpeg','-r500',[fpat,filesep,str]);
    %            print(myfigure,'-djpeg','-r100',[fpat,filesep,str]);
            end
        end
        
        if(fix_dt)
        else
            u_Norm2  = u .* u + v .* v;
            max_u    = sqrt( max( max(u_Norm2) ) );
            dt       = 0.9 * 1/max_u;
        end
    end
    
end